# encoding=utf8
# The ca lib & tools

"""The certificate authority lib & tool

This lib is based on openssl and follows the guide: https://jamielinux.com/docs/openssl-certificate-authority/

"""

from version import __version__

